<?php
/*
Plugin Name: WyPiekacz
Plugin URI: http://www.poradnik-webmastera.com/projekty/wypiekacz/
Description: Checks if posts submitted for review and posted satisfies set of rules.
Author: Daniel Frużyński
Version: 1.11
Author URI: http://www.poradnik-webmastera.com/
Text Domain: wypiekacz
*/

/*  Copyright 2009  Daniel Frużyński  (email : daniel [A-T] poradnik-webmastera.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


if ( !class_exists( 'WyPiekacz' ) || ( defined( 'WP_DEBUG') && WP_DEBUG ) ) {
	class WyPiekacz {
		var $errors = array();
		var $post_page = false;
		// Link counter - used by RX for removing links
		var $link_counter = 0;
		
		// Constructor
		function WyPiekacz() {
			// Save post - filter for data
			add_filter( 'wp_insert_post_data', array( &$this, 'wp_insert_post' ), 100, 2 );
			// Save post - called after post is saved
			add_action( 'save_post', array( &$this, 'save_post' ) );
			// Display notices in Admin panel
			add_action( 'admin_notices', array( &$this, 'admin_notice' ) );
			// Required to check if we are on Edit Post page
			add_action( 'do_meta_boxes', array( &$this, 'do_meta_boxes' ) );
			
			// Add new column on the Edit Posts page
			add_filter( 'manage_edit_columns', array( &$this, 'manage_edit_columns' ) );
			// Print values for new column on the Edit Posts page
			add_action( 'manage_posts_custom_column', array( &$this, 'manage_posts_custom_column' ), 10, 2 );
			
			// Default post template handling
			add_action( 'submitpost_box', array( &$this, 'submitpost_box' ) );
			
			// Add option to Admin menu
			add_action( 'admin_menu', array( &$this, 'admin_menu' ) );
			
			// Initialize plugin
			add_action( 'init', array( &$this, 'init' ) );
			add_action( 'admin_init', array( &$this, 'admin_init' ) );
		}
		
		// Plugin initialization
		function init() {
			if ( function_exists( 'load_plugin_textdomain' ) ) {
				load_plugin_textdomain( 'wypiekacz', PLUGINDIR.'/'.dirname(plugin_basename(__FILE__)) );
			}
		}
		
		// Plugin initialization - admin
		function admin_init() {
			// Register plugin options
			register_setting( 'wypiekacz', 'wypiekacz_min_len', array( &$this, 'sanitize_nonnegative' ) );
			register_setting( 'wypiekacz', 'wypiekacz_min_len_words', array( &$this, 'sanitize_nonnegative' ) );
			register_setting( 'wypiekacz', 'wypiekacz_max_links', array( &$this, 'sanitize_nonnegative' ) );
			register_setting( 'wypiekacz', 'wypiekacz_min_title_len', array( &$this, 'sanitize_nonnegative' ) );
			register_setting( 'wypiekacz', 'wypiekacz_max_title_len', array( &$this, 'sanitize_nonnegative' ) );
			register_setting( 'wypiekacz', 'wypiekacz_use_def_cat', array( &$this, 'sanitize_01' ) );
			register_setting( 'wypiekacz', 'wypiekacz_min_cats', array( &$this, 'sanitize_nonnegative' ) );
			register_setting( 'wypiekacz', 'wypiekacz_max_cats', array( &$this, 'sanitize_nonnegative' ) );
			register_setting( 'wypiekacz', 'wypiekacz_min_tags', array( &$this, 'sanitize_nonnegative' ) );
			register_setting( 'wypiekacz', 'wypiekacz_max_tags', array( &$this, 'sanitize_nonnegative' ) );
			register_setting( 'wypiekacz', 'wypiekacz_limit_for_all', array( &$this, 'sanitize_01' ) );
			register_setting( 'wypiekacz', 'wypiekacz_def_title' );
			register_setting( 'wypiekacz', 'wypiekacz_def_text' );
			register_setting( 'wypiekacz', 'wypiekacz_new_login_email', array( &$this, 'sanitize_01' ) );
			register_setting( 'wypiekacz', 'wypiekacz_pass_reset_email', array( &$this, 'sanitize_01' ) );
			register_setting( 'wypiekacz', 'wypiekacz_badwords', array( &$this, 'sanitize_stringlist' ) );
			register_setting( 'wypiekacz', 'wypiekacz_check_badwords_title', array( &$this, 'sanitize_01' ) );
			register_setting( 'wypiekacz', 'wypiekacz_check_badwords_content', array( &$this, 'sanitize_01' ) );
			register_setting( 'wypiekacz', 'wypiekacz_enforce_links', array( &$this, 'sanitize_01' ) );
			register_setting( 'wypiekacz', 'wypiekacz_enforce_title', array( &$this, 'sanitize_01' ) );
		}
		
		// Check if we are on Edit Post page
		function do_meta_boxes($type) {
			if ( $type == 'post' ) {
				$this->post_page = true;
			}
		}
		
		// Display notice in Admin panel
		function admin_notice() {
			global $post;
			$meta = '';
			if ( $this->post_page && is_object( $post) ) {
				$meta = get_post_meta($post->ID, 'WyPiekacz_msg', true);
			}
			if ('' != $meta ) {
				// Display error message
				echo '<div id="notice" class="error"><p>', $meta, 
					'<br />', __('Post Status has been changed to Draft.', 'wypiekacz'), '</p></div>', "\n";
				// Remove this message
				delete_post_meta( $post->ID, 'WyPiekacz_msg' );
				// Change WP message to 'Post saved'
				if ( isset( $_GET['message'] ) ) {
					if ( '6' == $_GET['message'] ) {
						$_GET['message'] = '7';
					}
				}
			}
		}
		
		// Check submitted post data
		function wp_insert_post( $data, $post_arr ) {
			if (
				// Check only for posts
				( 'post' == $data['post_type'])
				// Check only if status is Published or Pending Review
				&& ( ( 'publish' == $data['post_status'] ) || ( 'pending' == $data['post_status'] ) )
				// Editors (and above) can have limits too
				&& ( ( !get_option( 'wypiekacz_limit_for_all' ) && !current_user_can( 'edit_others_posts' ) )
					|| get_option( 'wypiekacz_limit_for_all' ) )
			) {
				$data = $this->enforce_rules( $data );
				
				$result = $this->check_precel_post( $data['post_content'], $data['post_title'], $post_arr );
				if ( true !== $result ) {
					// Revert post status to Draft
					$data['post_status'] = 'draft';
					
					// This is not supported in the core yet
					// See https://core.trac.wordpress.org/ticket/10480
					/*if ( defined( 'DOING_AJAX' ) && DOING_AJAX ) {
						$errors = $this->errors;
						$this->errors = array();
						return new WP_Error( 'edit_refused', implode( '; ', $errors ) );
					}*/
				}
			}
			return $data;
		}
		
		// Called after post is saved - save error messages too
		function save_post( $post_ID ) {
			if ( count( $this->errors ) > 0 ) {
				delete_post_meta( $post_ID, 'WyPiekacz_msg' );
				add_post_meta( $post_ID, 'WyPiekacz_msg', implode( '<br />', $this->errors ), true );
			}
		}
		
		// Add new column to the Edit Posts page
		function manage_edit_columns( $columns ) {
			if ( current_user_can( 'edit_others_posts' ) ) {
				$columns['wypiekacz'] = __('WyPiekacz Status', 'wypiekacz');
			}
			return $columns;
		}
		
		// Print values in new column on the Edit Posts page
		function manage_posts_custom_column( $column_name, $postID ) {
			if ( current_user_can( 'edit_others_posts' ) && $column_name === 'wypiekacz' ) {
				$post = get_post( $postID );
				if ( $this->check_precel_post( $post->post_content, $post->post_title, $post ) === true) {
					echo __('OK', 'wypiekacz');
				} else {
					echo '<span style="color:red">', implode( '<br />', $this->errors ), "</span>\n";
					$this->errors = array();
				}
			}
		}
		
		// Callback - remove extra links
		function rx_remove_links( $matches ) {
			++$this->link_counter;
			if ( $this->link_counter > get_option( 'wypiekacz_max_links' ) ) {
				return $matches[2];
			} else {
				return $matches[1].$matches[2].$matches[3];
			}
		}
		
		// Enforce rules for posts
		function enforce_rules( $data ) {
			// Enforce max link count
			if ( get_option( 'wypiekacz_enforce_links' ) ) {
				$this->link_counter = 0;
				$data['post_content'] = preg_replace_callback( '#(<a [^>]*href\s*=\s*[^>]+[^>]*>)(.*?)(</a>)#', 
					array( &$this, 'rx_remove_links' ), $data['post_content'] );
			}
			
			// Enforce max title length
			if ( get_option( 'wypiekacz_enforce_title' ) ) {
				$max_len = get_option( 'wypiekacz_max_title_len' );
				$len = strlen( $data['post_title'] );
				if ( $len > $max_len ) {
					$max_len -= 3; // Make room for three dots at the end
					if ( version_compare( PHP_VERSION, '5', '<' ) ) {
						$data['post_title'] = substr( $data['post_title'], 0, $max_len + 1 );
						$pos = strrpos( $data['post_title'], ' ' );
					} else {
						$pos = strrpos( $data['post_title'], ' ', $max_len - $len );
					}
					
					if ( $pos !== false ) {
						$data['post_title'] = substr( $data['post_title'], 0, $pos ) . '...';
					} else {
						// No spaces? Interesting...
						$data['post_title'] = substr( $data['post_title'], 0, $max_len ) . '...';
					}
				}
			}
			
			// TODO: Enforce max category count
			// Not supported in WP core yet
			/*if ( get_option( 'wypiekacz_enforce_cats' ) ) {
			}*/
			
			// TODO: Enforce max tag count
			// Not supported in WP core yet
			/*if ( get_option( 'wypiekacz_enforce_tags' ) ) {
			}*/
			
			return $data;
		}
		
		// Check if post obeys rules
		function check_precel_post( $text, $title, $post_data ) {
			// Check length (in characters)
			$min_len = get_option( 'wypiekacz_min_len' );
			$text2 = preg_replace( '/\s\s+/', ' ', trim( strip_tags( $text ) ) );
			$len = strlen( $text2 );
			if ( $len < $min_len ) {
				$this->errors[] = sprintf( __('Post is too short (minimum is %1$s chars, your post has %2$s).', 'wypiekacz'),
					$min_len, $len );
			}
			
			// Check length (in words)
			$min_len = get_option( 'wypiekacz_min_len_words' );
			$text2 = preg_replace( '/<.[^<>]*?>/', ' ', $text );
			$text2 = preg_replace( '/&nbsp;|&#160;/i', ' ', $text2 );
			$text2 = preg_replace( '/[0-9.(),;:!?%#$¿\'"_+=\\/-]*/', '', $text2 );
			$text2 = trim( $text2 );
			$count = preg_match_all( '/\S\s+/', $text2, $matches );
			if ( $count !== false ) {
				$count += 1;
			} elseif ( $tx != '' ) {
				$count = 1;
			} else {
				$count = 0;
			}
			if ( $count < $min_len ) {
				$this->errors[] = sprintf( __('Post is too short (minimum is %1$s words, your post has %2$s).', 'wypiekacz'),
					$min_len, $count );
			}
			
			// Check links
			$max_links = get_option( 'wypiekacz_max_links' );
			$cnt = preg_match_all( '/<\s*a\s/i', $text, $matches );
			if ( $cnt > $max_links ) {
				$this->errors[] = sprintf( __('Post contains too many links (maximum is %1$s, your post has %2$s).', 'wypiekacz'),
					$max_links, $cnt );
			}
			
			// Check title length
			$min_len = get_option( 'wypiekacz_min_title_len' );
			$max_len = get_option( 'wypiekacz_max_title_len' );
			$text2 = trim( $title );
			$len = strlen( $text2 );
			if ( $len < $min_len ) {
				$this->errors[] = sprintf( __('Post Title is too short (minimum is %1$s chars, your Title has %2$s).', 'wypiekacz'),
					$min_len, $len );
			}
			elseif ( $len > $max_len ) {
				$this->errors[] = sprintf( __('Post Title is too long (maximum is %1$s chars, your Title has %2$s).', 'wypiekacz'),
					$max_len, $len );
			}
			
			if ( is_array( $post_data ) ) {
				// Get categories from POST data
				if ( empty( $post_data['post_category'] ) || ( 0 == count( $post_data['post_category'] ) ) 
					|| !is_array( $post_data['post_category'] ) ) {
					$post_cat_cnt = 0;
					$categories = array();
				} else {
					$post_cat_cnt = count( $post_data['post_category'] );
					$categories = $post_data['post_category'];
				}
			} else {
				// Get categories from Post object
				$categories = wp_get_post_categories( $post_data->ID );
				$post_cat_cnt = count( $categories );
			}
			
			$has_default_cat = false;
			$default_cat = get_option( 'default_category' );
			foreach ( $categories as $category ) {
				if ( $category == $default_cat ) {
					$has_default_cat = true;
					break;
				}
			}
			
			// Check default category
			$use_default_cat = get_option( 'wypiekacz_use_def_cat' );
			if ( !$use_default_cat && $has_default_cat ) {
				$this->errors[] = sprintf( __('Cannot add posts to the default category (%s).', 'wypiekacz'), 
					get_cat_name( $default_cat ) );
			}
			
			// When post doesn't have categories, default one will be used
			if ( ( 0 == $post_cat_cnt ) && $use_default_cat ) {
				$post_cat_cnt = 1;
			}
			
			// Check categories count
			$min_cats = get_option( 'wypiekacz_min_cats' );
			$max_cats = get_option( 'wypiekacz_max_cats' );
			if ( $post_cat_cnt < $min_cats ) {
				$this->errors[] = sprintf( __('Too few categories selected (minimum is %1$s, your post has %2$s).', 'wypiekacz'),
					$min_cats, $post_cat_cnt );
			} else if ( $post_cat_cnt > $max_cats ) {
				$this->errors[] = sprintf( __('Too many categories selected (maximum is %1$s, your post has %2$s).', 'wypiekacz'),
					$max_cats, $post_cat_cnt );
			}
			
			if ( is_array( $post_data ) ) {
				//var_dump($post_data);die;
				$post_tag_cnt = 0;
				// Get tags from POST data
				if ( !empty( $post_data['adv-tags-input'] ) ) {
					// Simple Post Tags plugin uses this
					$tags = explode( ',', $post_data['adv-tags-input'] );
				} else if ( !empty( $post_data['tax_input'] ) && is_array( $post_data['tax_input'] ) ) {
					// WordPress 2.8+ uses general taxonomy support for tags too
					// Need to check this first, because $post_data['tags_input'] is set too
					
					// Check default tag taxonomy
					if ( isset( $post_data['tax_input']['post_tag'] ) ) {
						$tags = explode( ',', $post_data['tax_input']['post_tag'] );
					}
				} else if ( !empty( $post_data['tags_input'] ) ) {
					// Default WordPress field (up to 2.7.1)
					// Note: For some reason WP2.8 puts here an array when updating post
					if ( is_array( $post_data['tags_input'] ) ) {
						$tags = $post_data['tags_input'];
					} else {
						$tags = explode( ',', $post_data['tags_input'] );
					}
				} else {
					$tags = array();
				}
				
				foreach ($tags as $tag) {
					if ( trim( $tag ) != '' ) {
						++$post_tag_cnt;
					}
				}
			} else {
				// Get tags from Post object
				$tags = wp_get_post_tags( $post_data->ID );
				$post_tag_cnt = count( $tags );
			}
			
			
			// Check tag count
			$min_tags = get_option( 'wypiekacz_min_tags' );
			$max_tags = get_option( 'wypiekacz_max_tags' );
			
			if ( $post_tag_cnt < $min_tags ) {
				$this->errors[] = sprintf( __('Too few tags (minimum is %1$s, your post has %2$s).', 'wypiekacz'),
					$min_tags, $post_tag_cnt );
			} else if ($post_tag_cnt > $max_tags) {
				$this->errors[] = sprintf( __('Too many tags (maximum is %1$s, your post has %2$s).', 'wypiekacz'),
					$max_tags, $post_tag_cnt );
			}
			
			$badwords = get_option( 'wypiekacz_badwords', array() );
			$encoding = get_option( 'blog_charset' );
			$has_mb = function_exists( 'mb_convert_case' );
			
			// Check forbidden words in title
			if ( get_option( 'wypiekacz_check_badwords_title' ) ) {
				$words_found = array();
				if ( $has_mb ) {
					$title2 = mb_convert_case( $title, MB_CASE_LOWER, $encoding );
				}
				
				foreach ( $badwords as $badword ) {
					if ( $has_mb ) {
						$found = mb_strpos( $title2, $badword, 0, $encoding ) !== false;
					} else {
						$found = stripos( $title, $badword ) !== false;
					}
					
					if ( $found ) {
						$words_found[] = $badword;
					}
				}
				
				if ( count( $words_found ) > 0 ) {
					$this->errors[] = sprintf( __('Forbidden word(s) in title: %s', 'wypiekacz'),
						implode( ', ', $words_found ) );
				}
			}
			
			// Check forbidden words in content
			if ( get_option( 'wypiekacz_check_badwords_content' ) ) {
				$words_found = array();
				if ( $has_mb ) {
					$text2 = mb_convert_case( $text, MB_CASE_LOWER, $encoding );
				}
				
				foreach ( $badwords as $badword ) {
					if ( $has_mb ) {
						$found = mb_strpos( $text2, $badword, 0, $encoding ) !== false;
					} else {
						$found = stripos( $text, $badword ) !== false;
					}
					
					if ( $found ) {
						$words_found[] = $badword;
					}
				}
				
				if ( count( $words_found ) > 0 ) {
					$this->errors[] = sprintf( __('Forbidden word(s) in content: %s', 'wypiekacz'),
						implode( ', ', $words_found ) );
				}
			}
			
			return count( $this->errors ) == 0;
		}
		
		// Set Title and Content using defined template
		function submitpost_box() {
			global $post;
			
			// Title
			if ( empty( $post->post_title ) || ( '' == $post->post_title ) ) {
				$post->post_title = get_option( 'wypiekacz_def_title' );
			}
			
			// Text
			if ( empty( $post->post_content ) || ( '' == $post->post_content ) ) {
				$post->post_content = get_option( 'wypiekacz_def_text' );
			}
		}
		
		// Add Admin menu option
		function admin_menu() {
			$file = __FILE__;
			
			// hack for 1.5
			global $wp_version;
			if ( '1.5' == substr( $wp_version, 0, 3 ) ) {
				$file = 'wypiekacz/wypiekacz.php';
			}
			add_submenu_page( 'options-general.php', 'WyPiekacz', 
				'WyPiekacz', 10, $file, array( $this, 'options_panel' ) );
		}
		
		// Handle options panel
		function options_panel() {
			$message = null;
			if ( isset($_POST['action']) ) {
				check_admin_referer( 'wypiekacz-options' );
				$message = __('Configuration has been saved.', 'wypiekacz');
				echo '<div id="message" class="updated fade"><p>', $message, '</p></div>', "\n";
			}
?>
<div id="dropmessage" class="updated" style="display:none;"></div>
<div class="wrap">
<h2><?php _e('WyPiekacz - Options', 'wypiekacz'); ?></h2>

<form name="dofollow" action="options.php" method="post">
<?php settings_fields( 'wypiekacz' ); ?>
<table class="form-table">

<tr><th colspan="2"><h3><?php _e('Post:', 'wypiekacz'); ?></h3></th></tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_min_len"><?php _e('Minimum post length (in chars):', 'wypiekacz'); ?></label>
</th>
<td>
<input type="text" maxlength="6" size="10" id="wypiekacz_min_len" name="wypiekacz_min_len" value="<?php echo stripcslashes( get_option( 'wypiekacz_min_len' ) ); ?>" /><br /><?php _e('Note: if you want to check character count only, set minimum word count to zero.', 'wypiekacz'); ?>
</td>
</tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_min_len_words"><?php _e('Minimum post length (in words):', 'wypiekacz'); ?></label>
</th>
<td>
<input type="text" maxlength="6" size="10" id="wypiekacz_min_len_words" name="wypiekacz_min_len_words" value="<?php echo stripcslashes( get_option( 'wypiekacz_min_len_words' ) ); ?>" /><br /><?php _e('Note: if you want to check word count only, set minimum character count to zero.', 'wypiekacz'); ?>
</td>
</tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_max_links"><?php _e('Maximum link count in post:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="text" maxlength="2" size="10" id="wypiekacz_max_links" name="wypiekacz_max_links" value="<?php echo stripcslashes( get_option( 'wypiekacz_max_links' ) ); ?>" />
</td>
</tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_min_title_len"><?php _e('Minimum title length (in chars):', 'wypiekacz'); ?></label>
</th>
<td>
<input type="text" maxlength="3" size="10" id="wypiekacz_min_title_len" name="wypiekacz_min_title_len" value="<?php echo stripcslashes( get_option( 'wypiekacz_min_title_len' ) ); ?>" />
</td>
</tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_max_title_len"><?php _e('Maximum title length (in chars):', 'wypiekacz'); ?></label>
</th>
<td>
<input type="text" maxlength="3" size="10" id="wypiekacz_max_title_len" name="wypiekacz_max_title_len" value="<?php echo stripcslashes( get_option( 'wypiekacz_max_title_len' ) ); ?>" />
</td>
</tr>

<tr><th colspan="2"><h3><?php _e('Categories:', 'wypiekacz'); ?></h3></th></tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_use_def_cat"><?php printf( __('Posts can be added to the default category (%s):', 'wypiekacz'), get_cat_name( get_option( 'default_category' ) ) ); ?></label>
</th>
<td>
<input type="checkbox" id="wypiekacz_use_def_cat" name="wypiekacz_use_def_cat" value="yes" <?php checked( 1, get_option( 'wypiekacz_use_def_cat' ) ); ?> />
</td>
</tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_min_cats"><?php _e('Minimum categories selected:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="text" maxlength="3" size="10" id="wypiekacz_min_cats" name="wypiekacz_min_cats" value="<?php echo stripcslashes( get_option( 'wypiekacz_min_cats' ) ); ?>" />
</td>
</tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_max_cats"><?php _e('Maximum categories selected:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="text" maxlength="3" size="10" id="wypiekacz_max_cats" name="wypiekacz_max_cats" value="<?php echo stripcslashes( get_option( 'wypiekacz_max_cats' ) ); ?>" />
</td>
</tr>

<tr><th colspan="2"><h3><?php _e('Tags:', 'wypiekacz'); ?></h3></th></tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_min_tags"><?php _e('Minimum tag count:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="text" maxlength="3" size="10" id="wypiekacz_min_tags" name="wypiekacz_min_tags" value="<?php echo stripcslashes( get_option( 'wypiekacz_min_tags' ) ); ?>" />
</td>
</tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_max_tags"><?php _e('Maximum tag count:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="text" maxlength="3" size="10" id="wypiekacz_max_tags" name="wypiekacz_max_tags" value="<?php echo stripcslashes( get_option( 'wypiekacz_max_tags' ) ); ?>" />
</td>
</tr>

<tr><th colspan="2"><h3><?php _e('Forbidden words:', 'wypiekacz'); ?></h3></th></tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_check_badwords_title"><?php _e('Check for forbidden words in title:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="checkbox" id="wypiekacz_check_badwords_title" name="wypiekacz_check_badwords_title" value="yes" <?php checked( 1, get_option( 'wypiekacz_check_badwords_title' ) ); ?> />
</td>
</tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_check_badwords_content"><?php _e('Check for forbidden words in content:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="checkbox" id="wypiekacz_check_badwords_content" name="wypiekacz_check_badwords_content" value="yes" <?php checked( 1, get_option( 'wypiekacz_check_badwords_content' ) ); ?> />
</td>
</tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_badwords"><?php _e('Forbidden words list:', 'wypiekacz'); ?></label>
</th>
<td>
<textarea id="wypiekacz_badwords" name="wypiekacz_badwords" rows="5" cols="30"><?php echo htmlspecialchars( implode( 
"\n", get_option( 'wypiekacz_badwords', array() ) ) ); ?></textarea><br />
<?php _e('Put one word per line', 'wypiekacz'); ?>
</td>
</tr>

<tr><th colspan="2"><h3><?php _e('Rule enforcement:', 'wypiekacz'); ?></h3></th></tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_enforce_links"><?php _e('Enforce max links count:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="checkbox" id="wypiekacz_enforce_links" name="wypiekacz_enforce_links" value="yes" <?php checked( 1, get_option( 'wypiekacz_enforce_links' ) ); ?> /><br /><?php _e('Extra links over limit will be automatically removed', 'wypiekacz'); ?>
</td>
</tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_enforce_title"><?php _e('Enforce max title length:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="checkbox" id="wypiekacz_enforce_title" name="wypiekacz_enforce_title" value="yes" <?php checked( 1, get_option( 'wypiekacz_enforce_title' ) ); ?> /><br /><?php _e('Too long titles will be automatically truncated', 'wypiekacz'); ?>
</td>
</tr>

<tr><th colspan="2"><h3><?php _e('Special:', 'wypiekacz'); ?></h3></th></tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_limit_for_all"><?php _e('Check above rules for Editors and Administrators:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="checkbox" id="wypiekacz_limit_for_all" name="wypiekacz_limit_for_all" value="yes" <?php checked( 1, get_option( 'wypiekacz_limit_for_all' ) ); ?> />
</td>
</tr>

<tr><th colspan="2"><h3><?php _e('Default post template:', 'wypiekacz'); ?></h3></th></tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_def_title"><?php _e('Title:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="text" size="59" id="wypiekacz_def_title" name="wypiekacz_def_title" value="<?php echo stripcslashes( get_option( 'wypiekacz_def_title' ) ); ?>" />
</td>
</tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_def_text"><?php _e('Text:', 'wypiekacz'); ?></label>
</th>
<td>
<textarea rows="5" cols="57" id="wypiekacz_def_text" name="wypiekacz_def_text"><?php echo stripcslashes( get_option( 'wypiekacz_def_text' ) ); ?></textarea>
</td>
</tr>

<tr><th colspan="2"><h3><?php _e('Email notifications:', 'wypiekacz'); ?></h3></th></tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_new_login_email"><?php _e('Send notification of new login creation to admin:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="checkbox" id="wypiekacz_new_login_email" name="wypiekacz_new_login_email" value="yes" <?php checked( 1, get_option( 'wypiekacz_new_login_email' ) ); ?> />
</td>
</tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_pass_reset_email"><?php _e('Send notification of password resets to admin:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="checkbox" id="wypiekacz_pass_reset_email" name="wypiekacz_pass_reset_email" value="yes" <?php checked( 1, get_option( 'wypiekacz_pass_reset_email' ) ); ?> />
</td>
</tr>

</table>

<p class="submit">
<input type="hidden" name="action" value="update" />
<input type="submit" name="Submit" value="<?php _e('Save settings', 'wypiekacz'); ?>" /> 
</p>

</form>
</div>
<?php
		}
		
		// Sanitization functions
		function sanitize_01( $value ) {
			if ($value == 'yes') {
				$value = 1;
			} else {
				$value = (int)$value;
				if ( $value < 0 ) {
					$value = 0;
				} elseif ( $value > 1 ) {
					$value = 1;
				}
			}
			return $value;
		}
		
		function sanitize_nonnegative( $value ) {
			$value = (int)$value;
			if ( $value < 0 ) {
				$value = 0;
			}
			return $value;
		}
		
		function sanitize_stringlist( $value ) {
			$value = explode( "\n", (string)$value );
			$ret = array();
			$has_mb = function_exists( 'mb_convert_case' );
			$encoding = get_option( 'blog_charset' );
			foreach ( $value as $val ) {
				$val = trim( $val );
				if ( $val != '' ) {
					if ( $has_mb ) {
						$val = mb_convert_case( $val, MB_CASE_LOWER, $encoding );
					}
					if ( !in_array( $val, $ret ) ) {
						$ret[] = $val;
					}
				}
			}
			
			return $ret;
		}
	}
	
	add_option( 'wypiekacz_min_len', 1000 ); // Minimum post length (characters)
	add_option( 'wypiekacz_min_len_words', 0 ); // Minimum post length (words)
	add_option( 'wypiekacz_max_links', 3 ); // Maximum links per post
	add_option( 'wypiekacz_min_title_len', 5 ); // Minimum title length
	add_option( 'wypiekacz_max_title_len', 80 ); // Maximum title length
	add_option( 'wypiekacz_use_def_cat', 1 ); // Posts can be added to the default category
	add_option( 'wypiekacz_min_cats', 1 ); // Minimum category count
	add_option( 'wypiekacz_max_cats', 3 ); // Maximum category count
	add_option( 'wypiekacz_min_tags', 1 ); // Minimum tag count
	add_option( 'wypiekacz_max_tags', 5 ); // Maximum tag count
	add_option( 'wypiekacz_limit_for_all', 1 ); // Check ruled for Editors and Administrators
	add_option( 'wypiekacz_def_title', '' ); // Default post title
	add_option( 'wypiekacz_def_text', '' ); // Default post text
	add_option( 'wypiekacz_new_login_email', 1 ); // Send notification of new login creation to admin
	add_option( 'wypiekacz_pass_reset_email', 1 ); // Send notification of password resets to admin
	add_option( 'wypiekacz_badwords', array() ); // List of forbidden words
	add_option( 'wypiekacz_check_badwords_title', 0 ); // Check for forbidden words in title
	add_option( 'wypiekacz_check_badwords_content', 0 ); // Check for forbidden words in content
	add_option( 'wypiekacz_enforce_links', 0 ); // Enforce max link count
	add_option( 'wypiekacz_enforce_title', 0 ); // Enforce max title length
	
	$wp_wypiekacz = new WyPiekacz();
	
	// Redefine default WordPress functions in order to get control over sending extra emails to admin
	if ( !get_option('wypiekacz_new_login_email') && !function_exists( 'wp_new_user_notification' ) ) {
		// Notify user of his new username and password (skip admin)
		function wp_new_user_notification($user_id, $plaintext_pass = '') {
			if ( empty($plaintext_pass) )
				return;
			
			$user = new WP_User($user_id);
			
			$user_login = stripslashes($user->user_login);
			$user_email = stripslashes($user->user_email);
			
			$message  = sprintf(__('Username: %s'), $user_login) . "\r\n";
			$message .= sprintf(__('Password: %s'), $plaintext_pass) . "\r\n";
			$message .= wp_login_url() . "\r\n";
			
			@wp_mail($user_email, sprintf(__('[%s] Your username and password'), get_option('blogname')), $message);
		}
	}
	
	if ( !get_option('wypiekacz_pass_reset_email') && !function_exists( 'wp_password_change_notification' ) )
	{
		// Don't send password change notifications to admin
		function wp_password_change_notification(&$user) {
		}
	}
}

?>
